/* =============================================================================
 *
 * GlowingLED_Demo.c
 *
 * This file contains the code for the glowing LED demo task.
 *
 * v1.0	YFS	Initial release. 
 *
 * =============================================================================
*/

/*
* Copyright (2014), Cypress Semiconductor Corporation. All Rights Reserved.
*
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*/

/* Include FreeRTOS APIs and defines */
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <event_groups.h>

/* Include PSoC system and component APIs and defines */
#include <project.h>

/* Include application function declarations and defines */
#include <utils.h>
#include <App_Defs.h>
#include <Task_Defs.h>


/* Select a suitable PWM period - actual value is not critical */
#define PWM_PERIOD (250)

/*
 * Function:	GlowingLED_Demo_Task
 *
 * This function runs as a FreeRTOS task with a medium priority. It is a
 * simple demonstration of two PWMs driving an LED with varying intensity.
 *
 * Execution of this task is controlled by an event group. After creating
 * this task, the main task starts the demo by setting the bit in Demo_Events
 * that corresponds to this demo. When the user signals a desire to end the
 * demo, it stops the PWMs and then waits on the event bit again. This leaves
 * the task ready to be re-executed at a later date.
 * 
 *			Start task
 *		 ->	Check the demo stop flag. If so...
 *      |   	Wait on its event bit (sleep)
 *      |   	Clear LCD and print demo text
 *		|		(Re)Start the PWMs and choose LED outputs
 *		|		Clear the stop flag
 *		|	There is no C code in the application!
 *		|   Check if SW3 is pressed. If so...
 *		|   	Stop the PWMs and return the LEDs to software control
 *      |       Set the Main_Task event bit (make it ready to run)
 *		|   	Set the stop flag
 *		 -- Loop 
 *
 * Globals:		Uses Demo_Events and Main_Task_Event
 *
 * Parameters:	pvParameters (demo index number)
 *
 * Return:		None (infinite loop)
 */
 void GlowingLED_Demo_Task( void *pvParameters )
{
	int stop_demo = 1;	/* Flag for SW3 press (and first time into the code) */
	
	int demoNumber = (int)pvParameters;     /* Passed from xTaskCreate */
	
	for( ; ; )		/* Demo main loop (forever) */
	{
		/* Execute when task first starts and when the demo is started */
		if( stop_demo )
		{
            /* Wait on the demo event bit so that Main_Task can run */
            xEventGroupWaitBits(
                Demo_Events,            /* Wait for this demo to be requested */
                1 << demoNumber,        /* Event to wait - each demo owns a bit */
                pdTRUE,                 /* Clear the event on exit */
                pdFALSE,                /* Do not wait for all bits (irrelevant) */
                WAIT_FOREVER );
			
			/*
			MainTask posted to the TaskSem so it is time to run the demo again.
			Prepare the LCD, select the LED drivers and start the PWMs again.
			*/
			LCD_ClearDisplay();
			LCD_PrintString( GLOWINGLED_DEMO_TASK );
			
			/* Set the LEDs to be driven by the ANDed or XORed PWM outputs */
			LED3_Control_Set_Channel( LED3_Control_USE_IN1_AND_IN2 );
			LED4_Control_Set_Channel( LED4_Control_USE_IN1_XOR_IN2 );

			/* PWMs should have a 50% duty cycle and period off by one */
			PWM_1_WritePeriod( PWM_PERIOD );
			PWM_1_WriteCompare( PWM_PERIOD / 2 );
			PWM_1_Start();
	
			PWM_2_WritePeriod( PWM_PERIOD -1 );
			PWM_2_WriteCompare( PWM_PERIOD / 2 );
			PWM_2_Start();
			
			/* Clear the flag so we only run this once per demo run */
			stop_demo = 0;
			
		} /* if( stop_demo ) */	
			
			
		/* There is no application code in this loop - the demo is all HW */
		
		/* Check to see if SW3 was pressed - this signals the end of the demo */
		if( ! P15_5_Read() )		/* Note: 0 (ground) means SW3 was pressed */
		{
			/* Stop PWMs - good neighbor policy - another task may want them */
			PWM_1_Stop();
			PWM_2_Stop();
			
			LED3_Control_Set_Channel( LED3_Control_USE_SOFTWARE );
			LED4_Control_Set_Channel( LED4_Control_USE_SOFTWARE );
			
			/* Set the MainTask event bit to make it ready to run */
			xEventGroupSetBits( Main_Task_Event, MAIN_TASK_EVENT_NUM );
			
			/* Set flag to go to sleep (allow MainTask to run) */
			stop_demo = 1;

		} /* if( ! P15_5_Read() ) */

	} /* for( ; ; ) */

} /* GlowingLED_Demo_Task() */
